@@ -17,7 +17,9 @@
 # Standard library imports
 import inspect
 import logging
+from datetime import date
 
+import sqlalchemy
 # Related third party imports
 from sqlalchemy import (
     and_,
@@ -26,17 +28,46 @@ from sqlalchemy import (
     nullsfirst,
     nullslast,
     inspect as sqlalchemy_inspect,
+    text,
 )
 from sqlalchemy.ext.associationproxy import AssociationProxy
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import ColumnProperty
 from sqlalchemy.orm.attributes import InstrumentedAttribute, QueryableAttribute
+from sqlalchemy.sql.elements import BinaryExpression
+from sqlalchemy.sql.schema import Table as SQLAlchemySchemaTableType
 
 # Local application imports
-from faraday.server.models import User, CVE, Role
+from faraday.server.models import (
+    User,
+    CVE,
+    Role,
+    CustomFieldsSchema,
+    VulnerabilityGeneric,
+    Vulnerability,
+    VulnerabilityWeb,
+)
+
+VULNERABILITY_MODELS = (Vulnerability, VulnerabilityWeb, VulnerabilityGeneric)
 
 logger = logging.getLogger(__name__)
 
+bind_counter = 0
+
+
+def get_bind_counter():
+    return bind_counter
+
+
+def increment_bind_counter():
+    global bind_counter
+    bind_counter += 1
+
+
+def reset_bind_counter():
+    global bind_counter
+    bind_counter = 0
+
 
 def session_query(session, model):
     """Returns a SQLAlchemy query object for the specified `model`.
@@ -70,10 +101,18 @@ def get_related_association_proxy_model(attr):
 
 def primary_key_names(model):
     """Returns all the primary keys for a model."""
-    return [key for key, field in inspect.getmembers(model)
-            if isinstance(field, QueryableAttribute)
-            and isinstance(field.property, ColumnProperty)
-            and field.property.columns[0].primary_key]
+    return [
+        key for key, field in inspect.getmembers(model)
+        if isinstance(field, QueryableAttribute)
+        and (
+            isinstance(field.property, ColumnProperty) if hasattr(field, 'property') else False
+        )
+        and (
+            isinstance(field.property, ColumnProperty) if hasattr(field.property, 'columns') else False
+        )
+        and field.property.columns[0].primary_key
+        or isinstance(getattr(model, key, None), hybrid_property)
+    ]
 
 
 def _sub_operator(model, argument, fieldname):
@@ -97,8 +136,7 @@ def _sub_operator(model, argument, fieldname):
         relation = None
         if '__' in fieldname:
             fieldname, relation = fieldname.split('__')
-        return QueryBuilder._create_operation(submodel, fieldname, operator,
-                                              argument, relation)
+        return QueryBuilder._create_operation(submodel, fieldname, operator, argument, relation)
     # Support legacy has/any with implicit eq operator
     return getattr(submodel, fieldname) == argument
 
@@ -156,9 +194,69 @@ OPERATORS = {
     'has': lambda f, a, fn: f.has(_sub_operator(f, a, fn)),
     'any': lambda f, a, fn: f.any(_sub_operator(f, a, fn)),
     'not_any': lambda f, a, fn: ~f.any(_sub_operator(f, a, fn)),
+    # Range operator for numeric and date ranges
+    'range': lambda f, a: f.between(a[0], a[1]) if isinstance(a, (list, tuple)) and len(a) == 2 else f == a,
+    # Custom operator for json fields
+    'json': lambda f: f,
 }
 
 
+def get_json_operator(operator):
+    operator_mapping = {
+        '==': ('=', 'compare'),
+        'eq': ('=', 'compare'),
+        'equals': ('=', 'compare'),
+        'equal_to': ('=', 'compare'),
+        '!=': ('<>', 'compare'),
+        'ne': ('<>', 'compare'),
+        'neq': ('<>', 'compare'),
+        'not_equal_to': ('<>', 'compare'),
+        'does_not_equal': ('<>', 'compare'),
+        '<': ('<', 'compare'),
+        'lt': ('<', 'compare'),
+        '<=': ('<=', 'compare'),
+        'le': ('<=', 'compare'),
+        'lte': ('<=', 'compare'),
+        'leq': ('<=', 'compare'),
+        '>': ('>', 'compare'),
+        'gt': ('>', 'compare'),
+        '>=': ('>=', 'compare'),
+        'ge': ('>=', 'compare'),
+        'gte': ('>=', 'compare'),
+        'geq': ('>=', 'compare'),
+        'like': ('LIKE', 'compare'),
+        'ilike': ('ILIKE', 'compare'),
+        'any': ('@>', 'any'),
+        'not_any': ('@>', 'not_any'),
+        'is_null': ('IS NULL', 'exists'),
+        'is_not_null': ('IS NOT NULL', 'exists'),
+        'range': ('BETWEEN', 'range'),
+    }
+
+    return operator_mapping.get(operator, None)
+
+
+def get_json_query(table, field, op, op_type, counter):
+    if op_type == 'compare':
+        return f"({table}.{field} ->> :key_{counter}) {op} :value_{counter}"  # nosec
+    elif op_type == 'compare_int':
+        return f"({table}.{field} ->> :key_{counter})::int {op} :value_{counter}"  # nosec
+    elif op_type == 'exists':
+        return f"{table}.{field} ->> :key_{counter} {op}"  # nosec
+    elif op_type == 'any':
+        return f"{table}.{field} -> :key_{counter} {op} :value_{counter}"  # nosec
+    elif op_type == 'not_any':
+        return f"NOT {table}.{field} -> :key_{counter} {op} :value_{counter} OR {table}.{field} -> :key_{counter} IS NULL"  # nosec
+    elif op_type == 'list_contains':
+        return f"EXISTS (SELECT 1 FROM jsonb_array_elements_text({table}.{field} -> :key_{counter}) AS element WHERE element {op} :value_{counter})"  # nosec
+    elif op_type == 'date':
+        return f"({table}.{field} ->> :key_{counter})::DATE {op} :value_{counter}"  # nosec
+    elif op_type == 'range':
+        return f"({table}.{field} ->> :key_{counter})::numeric BETWEEN :value1_{counter} AND :value2_{counter}"  # nosec
+    else:
+        raise TypeError('Invalid filters')
+
+
 class OrderBy:
     """Represents an "order by" in a SQL query expression."""
 
@@ -342,7 +440,7 @@ class SearchParameters:
     def __repr__(self):
         """Returns a string representation of the search parameters."""
         template = ('<SearchParameters filters={0}, order_by={1}, limit={2},'
-                    ' group_by={3}, offset={4}, junction={5}>')
+                    ' group_by={3}, offset={4}>')
         return template.format(self.filters, self.order_by, self.limit,
                                self.group_by, self.offset)
 
@@ -442,30 +540,93 @@ class QueryBuilder:
           `relation` exists on `model`
 
         """
-        # raises KeyError if operator not in OPERATORS
-        opfunc = OPERATORS[operator]
-        # In Python 3.0 or later, this should be `inspect.getfullargspec`
-        # because `inspect.getargspec` is deprecated.
-        numargs = len(inspect.getargspec(opfunc).args)
-        # raises AttributeError if `fieldname` or `relation` does not exist
-        field = getattr(model, relation or fieldname)
-        # each of these will raise a TypeError if the wrong number of arguments
-        # is supplied to `opfunc`.
-        if numargs == 1:
-            return opfunc(field)
-        if argument is None:
-            msg = ('To compare a value to NULL, use the is_null/is_not_null '
-                   'operators.')
-            raise TypeError(msg)
-        if numargs == 2:
-            map_attr = {
-                'creator': 'username',
-            }
-            if hasattr(field, 'prop') and hasattr(getattr(field, 'prop'), 'entity'):
-                field = getattr(field.comparator.entity.class_, map_attr.get(field.prop.key, field.prop.key))
+        if '->' in fieldname:
+            if getattr(model, fieldname.split('->')[0]):
+                table = 'vulnerability' if model in VULNERABILITY_MODELS else model.__tablename__
 
-            return opfunc(field, argument)
-        return opfunc(field, argument, fieldname)
+                field, key = fieldname.split('->')
+                custom_field = CustomFieldsSchema.query.filter(CustomFieldsSchema.field_name == key).first()
+
+                try:
+                    op, op_type = get_json_operator(operator)
+                except TypeError as e:
+                    raise TypeError('Invalid filters') from e
+
+                if op in ['like', 'ilike']:
+                    if custom_field.field_type == 'list':
+                        op_type = 'list_contains'
+
+                if op_type in ['any', 'not_any']:
+                    if not argument.isnumeric():
+                        argument = f"\"{argument}\""
+
+                if custom_field.field_type == 'date':
+                    argument = date.fromisoformat(argument)
+                    op_type = 'date'
+
+                if op_type == 'compare':
+                    if custom_field.field_type == 'int':
+                        op_type = 'compare_int'
+
+                # Handle range operator
+                if op_type == 'range':
+                    # For JSON fields, argument should be a list [min, max]
+                    if not isinstance(argument, (list, tuple)) or len(argument) != 2:
+                        raise TypeError('Range operator requires a list or tuple with exactly two values')
+
+                    increment_bind_counter()
+
+                    bindparams = {
+                        f'key_{get_bind_counter()}': key,
+                        f'value1_{get_bind_counter()}': argument[0],
+                        f'value2_{get_bind_counter()}': argument[1],
+                    }
+
+                    query = get_json_query(table, field, op, op_type, get_bind_counter())
+
+                    return OPERATORS['json'](text(f"{query}").bindparams(**bindparams))
+
+                increment_bind_counter()
+
+                bindparams = {
+                    f'key_{get_bind_counter()}': key,
+                    f'value_{get_bind_counter()}': argument,
+                }
+
+                query = get_json_query(table, field, op, op_type, get_bind_counter())
+
+                return OPERATORS['json'](text(f"{query}").bindparams(**bindparams))
+        else:
+            # raises KeyError if operator not in OPERATORS
+            opfunc = OPERATORS[operator]
+            # In Python 3.0 or later, this should be `inspect.getfullargspec`
+            # because `inspect.getargspec` is deprecated.
+            # ╰─> Fixed that :)
+            try:
+                # For Python < 3.11
+                numargs = len(inspect.getargspec(opfunc).args)
+            except AttributeError:
+                # For Python >= 3.11
+                numargs = len(inspect.getfullargspec(opfunc).args)
+            # raises AttributeError if `fieldname` or `relation` does not exist
+            field = getattr(model, relation or fieldname)
+            # each of these will raise a TypeError if the wrong number of arguments
+            # is supplied to `opfunc`.
+            if numargs == 1:
+                return opfunc(field)
+            if argument is None:
+                msg = ('To compare a value to NULL, use the is_null/is_not_null '
+                       'operators.')
+                raise TypeError(msg)
+            if numargs == 2:
+                map_attr = {
+                    'creator': 'username',
+                }
+                if hasattr(field, 'prop') and hasattr(getattr(field, 'prop'), 'entity'):
+                    field = getattr(field.comparator.entity.class_, map_attr.get(field.prop.key, field.prop.key))
+
+                return opfunc(field, argument)
+            return opfunc(field, argument, fieldname)
 
     @staticmethod
     def _create_filter(model, filt):
@@ -505,12 +666,12 @@ class QueryBuilder:
         create_filt = QueryBuilder._create_filter
 
         def create_filters(filt):
-            if not isinstance(filt,
-                              JunctionFilter) and '__' in filt.fieldname:
+            if not isinstance(filt, JunctionFilter) and '__' in filt.fieldname:
                 return create_filt(model, filt)
             else:
-                if not getattr(filt, 'fieldname', False) or \
-                        filt.fieldname.split('__')[0] in valid_model_fields:
+                if (not getattr(filt, 'fieldname', False)
+                        or filt.fieldname.split('__')[0] in valid_model_fields
+                        or filt.fieldname.split('->')[0] in valid_model_fields):
                     try:
                         return create_filt(model, filt)
                     except AttributeError as e:
@@ -594,6 +755,17 @@ class QueryBuilder:
 
         filters = [filt for filt in filters_generator if filt is not None]
 
+        # Check if it is necessary to join the user's table
+        if model.__tablename__ != User.__tablename__:
+            for filter in filters:
+                if isinstance(filter, BinaryExpression):
+                    table = getattr(filter.left, "table", None)
+                    if not isinstance(table, SQLAlchemySchemaTableType):
+                        continue
+                    if table.name == User.__tablename__ and User not in joined_models:
+                        query = query.join(User, model.creator_id == User.id)
+                        joined_models.add(User)
+
         # Multiple filter criteria at the top level of the provided search
         # parameters are interpreted as a conjunction (AND).
         query = query.filter(*filters)
@@ -659,6 +831,66 @@ class QueryBuilder:
 
         return query
 
+    @staticmethod
+    def create_query_delete_returning_only_ids(session, model, search_params, _ignore_order_by=False):
+
+        query = sqlalchemy.delete(model)
+
+        # This function call may raise an exception.
+        valid_model_fields = []
+        for orm_descriptor in sqlalchemy_inspect(model).all_orm_descriptors:
+            if isinstance(orm_descriptor, InstrumentedAttribute):
+                valid_model_fields.append(str(orm_descriptor).split('.')[1])
+            if isinstance(orm_descriptor, hybrid_property):
+                valid_model_fields.append(orm_descriptor.__name__)
+        valid_model_fields += [str(algo).split('.')[1] for algo in sqlalchemy_inspect(model).relationships]
+
+        filters_generator = map(  # pylint: disable=W1636
+            QueryBuilder.create_filters_func(model, valid_model_fields),
+            search_params.filters
+        )
+
+        filters = [filt for filt in filters_generator if filt is not None]
+
+        # Multiple filter criteria at the top level of the provided search
+        # parameters are interpreted as a conjunction (AND).
+        query = query.where(*filters)
+
+        return query
+
+    @staticmethod
+    def create_query_only_ids(session, model, search_params, _ignore_order_by=False):
+
+        query = session.query(model.id)
+
+        # This function call may raise an exception.
+        valid_model_fields = []
+        for orm_descriptor in sqlalchemy_inspect(model).all_orm_descriptors:
+            if isinstance(orm_descriptor, InstrumentedAttribute):
+                valid_model_fields.append(str(orm_descriptor).split('.')[1])
+            if isinstance(orm_descriptor, hybrid_property):
+                valid_model_fields.append(orm_descriptor.__name__)
+        valid_model_fields += [str(algo).split('.')[1] for algo in sqlalchemy_inspect(model).relationships]
+
+        filters_generator = map(  # pylint: disable=W1636
+            QueryBuilder.create_filters_func(model, valid_model_fields),
+            search_params.filters
+        )
+
+        filters = [filt for filt in filters_generator if filt is not None]
+
+        # Multiple filter criteria at the top level of the provided search
+        # parameters are interpreted as a conjunction (AND).
+        query = query.filter(*filters)
+
+        # Apply limit and offset to the query.
+        if search_params.limit:
+            query = query.limit(search_params.limit)
+        if search_params.offset:
+            query = query.offset(search_params.offset)
+
+        return query
+
 
 def create_query(session, model, search_params, _ignore_order_by=False):
     """Returns a SQLAlchemy query object on the given `model` where the search
@@ -724,6 +956,37 @@ def search(session, model, search_params, _ignore_order_by=False):
     # False (False, 0, the empty string, the empty list, etc.).
     is_single = search_params.get('single')
     query = create_query(session, model, search_params, _ignore_order_by)
+    if is_single:
+        # may raise NoResultFound or MultipleResultsFound
+        return query.one()
+    reset_bind_counter()
+    return query
+
+
+def create_query_retrieve_only_ids(session, model, search_params):
+    if isinstance(search_params, dict):
+        search_params = SearchParameters.from_dictionary(search_params)
+    return QueryBuilder.create_query_only_ids(session, model, search_params)
+
+
+def search_retrieve_only_ids(session, model, search_params):
+    is_single = search_params.get('single')
+    query = create_query_retrieve_only_ids(session, model, search_params)
+    if is_single:
+        # may raise NoResultFound or MultipleResultsFound
+        return query.one()
+    return query
+
+
+def create_query_delete_returning_only_ids(session, model, search_params):
+    if isinstance(search_params, dict):
+        search_params = SearchParameters.from_dictionary(search_params)
+    return QueryBuilder.create_query_delete_returning_only_ids(session, model, search_params)
+
+
+def delete_returning_only_ids(session, model, search_params):
+    is_single = search_params.get('single')
+    query = create_query_delete_returning_only_ids(session, model, search_params)
     if is_single:
         # may raise NoResultFound or MultipleResultsFound
         return query.one()
